package com.imin.scandemo;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.content.res.Resources;
import android.graphics.ImageFormat;
import android.hardware.camera2.CameraAccessException;
import android.hardware.camera2.CameraCaptureSession;
import android.hardware.camera2.CameraCharacteristics;
import android.hardware.camera2.CameraDevice;
import android.hardware.camera2.CameraManager;
import android.hardware.camera2.CaptureRequest;
import android.media.Image;
import android.media.ImageReader;
import android.os.Build;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Bundle;
import android.util.Log;
import android.util.SparseIntArray;
import android.view.Surface;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.widget.ImageView;
import android.widget.TextView;
import androidx.core.app.ActivityCompat;

import com.imin.scan.Result;
import com.imin.scan.ScanUtils;
import com.imin.scan.Symbol;

import java.nio.ByteBuffer;
import java.util.Arrays;

public class MainActivity3 extends Activity {
    private static final SparseIntArray ORIENTATIONS = new SparseIntArray();
    private static final String TAG = "MainActivity3" ;

    ///为了使照片竖直显示
    static {
        ORIENTATIONS.append(Surface.ROTATION_0, 90);
        ORIENTATIONS.append(Surface.ROTATION_90, 0);
        ORIENTATIONS.append(Surface.ROTATION_180, 270);
        ORIENTATIONS.append(Surface.ROTATION_270, 180);
    }

    private SurfaceView mSurfaceView;
    private SurfaceHolder mSurfaceHolder;
    private ImageView iv_show;
    private CameraManager mCameraManager;//摄像头管理器
    private Handler childHandler, mainHandler;
    private String mCameraID;//摄像头Id 0 为后  1 为前
    private ImageReader mImageReaderJPG;
    private ImageReader mImageReaderPreview;
    private CameraCaptureSession mCameraCaptureSession;
    private CameraDevice mCameraDevice;

    //预览分辨率设置，T1/T2 mini设置640x480，其他手持机可选取640x480,800x480,1280x720
    public static int previewSize_width=640;
    public static int previewSize_height=480;
    public static int previewSize_width_pro=960;
    public static int previewSize_height_pro=720;
    StringBuilder sb = new StringBuilder();
    String content="";
    public int decode_count = 0;
    private TextView textview;
    private ScanUtils scanUtils;



    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main3);
        initVIew();
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    /**
     * 初始化
     */
    private void initVIew() {
        //mSurfaceView
        mSurfaceView = (SurfaceView) findViewById(R.id.surface_view);
        textview = (TextView) findViewById(R.id.textview);
        mSurfaceHolder = mSurfaceView.getHolder();
        mSurfaceHolder.setKeepScreenOn(true);
        // mSurfaceView添加回调
        mSurfaceHolder.addCallback(new SurfaceHolder.Callback() {
            @Override
            public void surfaceCreated(SurfaceHolder holder) { //SurfaceView创建
                // 初始化Camera
                initCamera2();
            }

            @Override
            public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
            }

            @Override
            public void surfaceDestroyed(SurfaceHolder holder) { //SurfaceView销毁
                // 释放Camera资源
                if (null != mCameraDevice) {
                    mCameraDevice.close();
                    MainActivity3.this.mCameraDevice = null;
                }
            }
        });

        //初始化扫码工具
        //获取解码工具类
        scanUtils = ScanUtils.getInstance(this);
        scanUtils.initScan();//初始化
        scanUtils.initBeepSound(true,R.raw.beep);//初始化音频文件
        scanUtils.setConfig(Symbol.ALL_FORMATS);//允许所有的解码类型
    }


    /**
     * 初始化Camera2
     */

    private void initCamera2() {
        HandlerThread handlerThread = new HandlerThread("Camera2");
        handlerThread.start();
        childHandler = new Handler(handlerThread.getLooper());
        mainHandler = new Handler(getMainLooper());
        if (Build.MODEL.equals("M2-Pro")) {
            mCameraID = "" + CameraCharacteristics.LENS_FACING_FRONT;//LENS_FACING_FRONT;//后摄像头
            mImageReaderPreview = ImageReader.newInstance(previewSize_width_pro, previewSize_height_pro, ImageFormat.YUV_420_888,1);
        }else{
            mCameraID = "" + CameraCharacteristics.LENS_FACING_BACK;//LENS_FACING_FRONT;//后摄像头
            mImageReaderPreview = ImageReader.newInstance(previewSize_width, previewSize_height, ImageFormat.YUV_420_888,1);
        }

        if(Build.MODEL.equals("D1") || Build.MODEL.equals("D1-Pro") || Build.MODEL.equals("I22M01")){
            mCameraID = "" + CameraCharacteristics.LENS_FACING_FRONT;//LENS_FACING_FRONT;//后摄像头
        }

        mImageReaderPreview.setOnImageAvailableListener(new ImageReader.OnImageAvailableListener() { //可以在这里处理拍照得到的临时照片 例如，写入本地
            @Override
            public void onImageAvailable(ImageReader reader) {
                Image image = reader.acquireLatestImage();
                //我们可以将这帧数据转成字节数组，类似于Camera1的PreviewCallback回调的预览帧数据
                ByteBuffer buffer = image.getPlanes()[0].getBuffer();
                byte[] data = new byte[buffer.remaining()];
                buffer.get(data);

                Result rawResult = scanUtils.getScanResult(data,previewSize_width,previewSize_height);
                if (scanUtils.getNsyms() != 0 && rawResult != null) {
                    Resources resources = getResources();
                    sb.append(resources.getString(R.string.count) + decode_count++ );
                    sb.append("\n"+resources.getString(R.string.time_consuming) + scanUtils.getCost_time() + " ms\n" );
//				//如果允许识读多个条码，则解码结果可能不止一个
                    sb.append(resources.getString(R.string.symbology) + rawResult.getNumBits()+ "\n");
                    sb.append(resources.getString(R.string.capacity) + rawResult.getBarcodeFormat().name()+ "\n");
                    sb.append(resources.getString(R.string.content) + rawResult.toString());
                }
                Log.i("XGH",":"+sb.toString());
                textview.setText(sb.toString());
                sb.delete(0,sb.length());
                image.close();
            }
        }, null);
        //获取摄像头管理
        mCameraManager = (CameraManager) getSystemService(Context.CAMERA_SERVICE);
        try {
            if (ActivityCompat.checkSelfPermission(this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {
                return;
            }
            //打开摄像头
            mCameraManager.openCamera(mCameraID, stateCallback, mainHandler);

        } catch (CameraAccessException e) {
            e.printStackTrace();
        }
    }


    /**
     * 摄像头创建监听
     */
    private CameraDevice.StateCallback stateCallback = new CameraDevice.StateCallback() {
        @Override
        public void onOpened(CameraDevice camera) {//打开摄像头
            mCameraDevice = camera;
            //开启预览
            takePreview();
        }

        @Override
        public void onDisconnected(CameraDevice camera) {//关闭摄像头
            if (null != mCameraDevice) {
                mCameraDevice.close();
                MainActivity3.this.mCameraDevice = null;
            }
        }

        @Override
        public void onError(CameraDevice camera, int error) {//发生错误
            //Toast.makeText(MainActivity3.this, "摄像头开启失败", Toast.LENGTH_SHORT).show();
        }
    };

    /**
     * 开始预览
     */
    private void takePreview() {
        try {
            // 创建预览需要的CaptureRequest.Builder
            final CaptureRequest.Builder previewRequestBuilder = mCameraDevice.createCaptureRequest(CameraDevice.TEMPLATE_PREVIEW);
            // 将SurfaceView的surface作为CaptureRequest.Builder的目标
            previewRequestBuilder.addTarget(mSurfaceHolder.getSurface());
            previewRequestBuilder.addTarget(mImageReaderPreview.getSurface());
            // 创建CameraCaptureSession，该对象负责管理处理预览请求和拍照请求
            mCameraDevice.createCaptureSession(Arrays.asList(mSurfaceHolder.getSurface(), mImageReaderPreview.getSurface()), new CameraCaptureSession.StateCallback() {
                @Override
                public void onConfigured(CameraCaptureSession cameraCaptureSession) {
                    if (null == mCameraDevice) return;
                    // 当摄像头已经准备好时，开始显示预览
                    mCameraCaptureSession = cameraCaptureSession;
                    try {
                        // 自动对焦
                        previewRequestBuilder.set(CaptureRequest.CONTROL_AF_MODE, CaptureRequest.CONTROL_AF_MODE_CONTINUOUS_PICTURE);
                        // 打开闪光灯
                        previewRequestBuilder.set(CaptureRequest.CONTROL_AE_MODE, CaptureRequest.CONTROL_AE_MODE_ON_AUTO_FLASH);
                        // 显示预览
                        CaptureRequest previewRequest = previewRequestBuilder.build();
                        mCameraCaptureSession.setRepeatingRequest(previewRequest, null, childHandler);
                    } catch (CameraAccessException e) {
                        e.printStackTrace();
                    }
                }

                @Override
                public void onConfigureFailed(CameraCaptureSession cameraCaptureSession) {
                    //Toast.makeText(MainActivity3.this, "配置失败", Toast.LENGTH_SHORT).show();
                }
            }, childHandler);
        } catch (CameraAccessException e) {
            e.printStackTrace();
        }
    }


    @Override
    protected void onDestroy() {
        super.onDestroy();
        if ( scanUtils!= null) {
            scanUtils.destroy();
        }
    }
}
